import { Router, RouteLocationRaw, RouteParamsRaw, LocationQueryRaw, RouterOptions, RouteLocationNormalizedLoaded, RouteLocationMatched } from 'vue-router';
import * as vue from 'vue';
import { Ref, ComputedRef } from 'vue';
import { VueWrapper } from '@vue/test-utils';

declare const EmptyView: vue.DefineComponent<{}, {}, {}, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, vue.EmitsOptions, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<{} & {} & {}>, {}>;
/**
 * Router Mock instance
 */
interface RouterMock extends Router {
    /**
     * Current depth of the router view. This index is used to find the component
     * to display in the array `router.currentRoute.value.matched`.
     */
    depth: Ref<number>;
    /**
     * Set a value to be returned on a navigation guard for the next navigation.
     *
     * @param returnValue - value that will be returned on a simulated navigation
     * guard
     */
    setNextGuardReturn(returnValue: Error | boolean | RouteLocationRaw | undefined): void;
    /**
     * Returns a Promise of the pending navigation. Resolves right away if there
     * isn't any.
     */
    getPendingNavigation(): ReturnType<Router['push']>;
    /**
     * Sets the params of the current route without triggering a navigation. Can
     * be awaited to wait for Vue to render again.
     *
     * @param params - params to set in the current route
     */
    setParams(params: RouteParamsRaw): Promise<void>;
    /**
     * Sets the query of the current route without triggering a navigation. Can
     * be awaited to wait for Vue to render again.
     *
     * @param query - query to set in the current route
     */
    setQuery(query: LocationQueryRaw): Promise<void>;
    /**
     * Sets the hash of the current route without triggering a navigation. Can
     * be awaited to wait for Vue to render again.
     *
     * @param hash - hash to set in the current route
     */
    setHash(hash: string): Promise<void>;
    /**
     * Clear all the mocks and reset the location of the router. This is useful to be called in a `beforeEach()` test hook
     * to reset the router state before each test.
     */
    reset(): void;
}
/**
 * TODO: Allow passing a custom spy and detect common global ones like jest and cypress.
 */
/**
 * Options passed to `createRouterMock()`.
 */
interface RouterMockOptions extends Partial<RouterOptions> {
    /**
     * Override the starting location before each test. Defaults to
     * START_LOCATION.
     */
    initialLocation?: RouteLocationRaw;
    /**
     * Run in-component guards. Defaults to false. Setting this to `true` will also run global guards as if
     * `useRealNavigation` was set to `true`.
     */
    runInComponentGuards?: boolean;
    /**
     * Runs all navigation through a `push()` or `replace()` to effectively run any global.
     */
    useRealNavigation?: boolean;
    /**
     * Run per-route guards. Defaults to false.
     * @deprecated use `removePerRouteGuards` instead
     */
    runPerRouteGuards?: boolean;
    /**
     * Removes `beforeEnter` guards to any route added. Defaults to `true`.
     */
    removePerRouteGuards?: boolean;
    /**
     * By default the mock will allow you to push to locations without adding all
     * the necessary routes so you can still check if `router.push()` was called
     * in a specific scenario
     * (https://github.com/posva/vue-router-mock/issues/41). Set this to `true` to
     * disable that behavior and throw when `router.push()` fails.
     */
    noUndeclaredRoutes?: boolean;
}
/**
 * Creates a router mock instance
 *
 * @param options - options to initialize the router
 */
declare function createRouterMock(options?: RouterMockOptions): RouterMock;

/**
 * Inject global variables, overriding any previously inject router mock
 *
 * @param router - router mock to inject
 */
declare function injectRouterMock(router?: RouterMock): {
    router: RouterMock;
    route: RouteLocationNormalizedLoaded;
};
/**
 * Creates an object of properties to be provided at your application level to
 * mock what is injected by vue-router
 *
 * @param router - router mock instance
 */
declare function createProvide(router: RouterMock): {
    [x: number]: RouteLocationNormalizedLoaded | Ref<RouteLocationNormalizedLoaded> | RouterMock | ComputedRef<RouteLocationMatched>;
};

declare function plugin(wrapper: VueWrapper<any>): VueWrapper<any>;
declare function getRouter(): RouterMock;

export { EmptyView, RouterMock, RouterMockOptions, plugin as VueRouterMock, createProvide, createRouterMock, getRouter, injectRouterMock };
